#!/opt/perl/bin/perl
#
# This script is used to extract trap definitions from a
# mib file. It works by using a finite state machine 
# to recognize ASN.1 trap definitions. The trap definition
# can either be V1 TRAP-TYPE or V2 NOTIFICATION-TYPE.
#
# The output generated is used in specialized scripts.
#
# Authior: Andy Yuen
# Date: August 2005.
#
##################################################
package MIBFSM;

# parser state machine
#
my $COMMENT_ID = 0;
my $NAME_ID = 1;
my $NUM_ID = 2;

# token Id used for recognizing trap definitions
my %ID = (
	"--"				=>	0,
	"NOTIFICATION-TYPE"	=>	3,
	"TRAP-TYPE"			=>	4,
	"OBJECTS"			=>	5,
	"VARIABLES"			=>	5,
	"{"					=>	6,
	"}"					=>	7,
	"::="				=>	8
);

# finite state machine for recognizing trap definitions
my @FSM = (
	#--,name,	num,	n-type,	t-type,	OBJECTS,	{,	},	::=
	[0,	0,		0,		1,		9,		0,			0,	0,	0,],	# state 0
	[1,	0,		0,		0,		0,		2,			0,	0,	0,],	#		1
	[2,	0,		0,		0,		0,		0,			3,	0,	0,],	#		2
	[3,	3,		0,		0,		0,		0,			0,	4,	0,],	#		3
	[4,	4,		4,		4,		4,		4,			4,	4,	5,],	#		4
	[5,	0,		8,		0,		0,		0,			6,	0,	0,],	#		5
	[6,	7,		0,		0,		0,		0,			0,	0,	0,],	#		6
	[7,	0,		8,		0,		0,		0,			0,	0,	0,],	#		7
	[8,	0,		0,		0,		0,		0,			0,	0,	0,],	#		8
	[9,	9,		0,		0,		0,		2,			0,	0,	0,]		#		9
);


# constructor
sub new {
	my $class = shift;
    my $self = {
		'tokens'		=> [],
		'trapnum'		=> [],
		'trapname'		=> [],
		'pos'			=> [],
		'vars'			=> [],
		'delimiter'		=> "~"
	};
  
	bless $self, $class;
  
	return $self;
}


# get mib info
sub getMibInfo {
	my ($self, $filename, $mibVar, $str) = @_;

	local *INPUT;
	open(INPUT, $filename) || die "Could not open file $filename: $!\n";

	my $pos = -1;
	my $token;
	my $id;
	my $trapname;
	my $trapnum;
	my @vars = ();
	my $state = 0;

	$self->_reset;
	# process mib file as a sequence of tokens
	$token = $self->_getToken();
	while ($token ne "") {
		$id = $self->_getTokenId($token);
		#printf "State=%d Token: %s Id=%d\n", $state, $token, $id;

		# perform actions associated with each state and input
		if ($id == $COMMENT_ID) {
			# comment starts here, discard tokens
			$self->_discardTokens();
		}
		else {
			if ((($state == 0) || ($state == 8)) && ($id == $NAME_ID)) {
				# capture possible trap name
				$trapname = $token;
				@vars = ();
			}
			elsif (($state == 3) && ($id == $NAME_ID)) {
				# capture varbind name
				my $val = $token;
				if ($str) {
					# remove substring in varbind name if command line specifies a substring to remove
					$val =~ s/$str//;
				}
				push @vars, $val;
				if ($mibVar) {
					# record position of varbind if command line specifies a variable to track
					if ($mibVar eq $token) {
						$pos = $#vars;
					}
				}
				#printf "Varbind: %s\n", $token;
			}
			elsif ((($state == 7) || ($state == 5)) && ($id == $NUM_ID)) {
				# capture trap number
				$trapnum = $token;
			}
			
			$state = $FSM[$state][$id];

			if ($state == 8) {
				# output trap info
				#printf "Trap %s: %s\n", $trapnum, $trapname;
				#foreach my $var (@vars) {
				#	printf "\t%s\n", $var;
				#}
				push @{$self->{'trapnum'}}, $trapnum;
				push @{$self->{'trapname'}}, $trapname;
				push @{$self->{'vars'}}, join($self->{'delimiter'}, @vars);
				if ($mibVar) {
					#printf qq("%s" => [%s,\n\t%d,\n\t"%s"],\n), $trapnum, $trapname, $pos, join("=%s, ", @vars) . "=%s";
					push @{$self->{'pos'}}, $pos;

				}
				else {
					#printf qq("%s" => [%s,\n\t"%s"],\n), $trapnum, $trapname, join("=%s, ", @vars) . "=%s";
				}
				$pos = -1;
			}
		}
		$token = $self->_getToken();
	}

	# clean up
	close(INPUT);

}

# get total trap definitions
sub getTotalTraps {
	my ($self) = @_;
	return scalar(@{$self->{'trapnum'}});
}

# get trapnum indexed
sub getTrapNum {
	my ($self, $index) = @_;
	return $self->{'trapnum'}->[$index];
}

# get trapname indexed
sub getTrapName {
	my ($self, $index) = @_;
	return $self->{'trapname'}->[$index];
}
# get varpos indexed
sub getVarPos {
	my ($self, $index) = @_;
	return $self->{'pos'}->[$index];
}
# get vars indexed
sub getVars {
	my ($self, $index) = @_;
	return $self->{'vars'}->[$index];
}

# get tokens in a mib file one line at a time
sub _getTokens {
	my ($self) = @_;
	while () {
		my $line = <INPUT>;
		#printf "Line: %s\n", $line;
		if ($line) {
			chomp($line);
			if ($line eq "") {
				next;
			}
			# note that Perl's split is not as powerful as that of awk
			# can't have a dynamic regular expression
			return 0, split /[ \t,]/, $line;
		}
		#print "End-of-file\n";
		return 1, undef;
	}
}

# private: return the next token in the line just tokenized
sub _getToken {
	my ($self) = @_;
	while (1) {
		if (scalar(@{$self->{'tokens'}}) > 0) {
			my $value = shift @{$self->{'tokens'}};
			# skip empty string
			if ($value ne "") {
				# handling cases where {is followed immediately by a name and } immediately follows a name
				if (length($value) > 1) {
					if (substr($value, 0, 1) eq '{') {
						unshift @{$self->{'tokens'}}, substr($value, 1);
						$value = '{';
					}
					elsif (substr($value, -1, 1) eq '}') {
						unshift @{$self->{'tokens'}}, "}";
						$value = substr($value, 0, -1);
					}
				}
				return $value;
			}
		}
		else {
			my $nomore;
			($nomore, @{$self->{'tokens'}}) = $self->_getTokens();
			if ($nomore == 1) {
				#print "_getToken: End-of-file\n";
				return "";
			}
		}
	}
}

# private: return the token Id associated with the given token
sub _getTokenId {
	my ($self, $token) = @_;

	if (exists($ID{$token})) {
		return $ID{$token};
	}

	if ($token =~ /^\d+$/) {
		return $NUM_ID;
	}

	return $NAME_ID;
}

# private: discard all tokens from the most recently parsed line
sub _discardTokens {
	my ($self) = @_;
	@{$self->{'tokens'}} = ();
}

# private: reset all instance variables
sub _reset {
	my ($self) = @_;
	@{$self->{'tokens'}} = ();
	@{$self->{'trapnum'}} = ();
	@{$self->{'trapname'}} = ();
	@{$self->{'pos'}} = ();
	@{$self->{'vars'}} = ();
}

1;